package com.icontrols.kakao.controller;

import javax.servlet.http.HttpServletResponse;

import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.hateoas.PagedResources;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.server.ResponseStatusException;

import com.icontrols.kakao.utils.JWTUtils;
import com.sun.net.httpserver.HttpServer;

@RestController
public class MainController {

	public String COMPLEX_SERVER = "http://61.33.215.54:8003/kakao/danzi";
	String KAKAO_SERVER_EVENT = "https://kakaohome-api.i.kakao.com/events";
//	String KAKAO_SERVER_EVENT = "https://kakaohome-api-beta.i.kakao.com/events";
	String KAKAO_SERVER_EVENT_SANDBOX = "https://kakaohome-api-sandbox.i.kakao.com/events";

	private static final Logger logger = LoggerFactory.getLogger(MainController.class);

	@RequestMapping(value = "/kakao", method = RequestMethod.POST, consumes = "application/json", produces = "application/json")
	public @ResponseBody String From3rdServer(@RequestHeader HttpHeaders headers, HttpEntity<String> httpEntity,
			HttpServletResponse response) {
		logger.info("/kakao");
		String answer = "";
		String body = httpEntity.getBody();
		logger.info("Body: " + body);
		JSONObject obj = new JSONObject(body);
		String token = obj.get("authorization").toString();

		JWTUtils jwt = new JWTUtils();
		String complexIp = jwt.getCmpIpFromToken(token);
		logger.info(complexIp);
		String complexServer = "http://" + complexIp + ":8003/kakao/danzi";

		HttpComponentsClientHttpRequestFactory factory = new HttpComponentsClientHttpRequestFactory();
		factory.setConnectTimeout(3000);
		RestTemplate template = new RestTemplate(factory);

		response.setHeader("Content-Type", "application/json");
//		response.setHeader("Authorization", headers.get("Authorization").get(0));
		HttpEntity<String> entity = new HttpEntity<String>(body, headers);
		answer = template.postForObject(complexServer, entity, String.class);

		return answer;
	}

	@RequestMapping(value = "/events", method = RequestMethod.POST, consumes = "application/json", produces = "application/json")
	public @ResponseStatus @ResponseBody ResponseEntity<?> FromValleyServerEvent(@RequestHeader HttpHeaders headers,
			HttpEntity<String> httpEntity, HttpServletResponse response) {

		logger.info("/events");

		// beta version
//		String KAKAO_SERVER_EVENT = "https://kakaohome-api-beta.i.kakao.com/events";
		String answer = "";
		String body = httpEntity.getBody();
		logger.info("Body: " + body);
		JSONObject obj = new JSONObject(body);
		String token = obj.get("authorization").toString();
//		JWTUtils jwt = new JWTUtils();
//		String complexIp = jwt.getCmpIpFromToken(token);
//		logger.info(complexIp);
		CloseableHttpClient httpClient = HttpClients.custom().setSSLHostnameVerifier(new NoopHostnameVerifier())
				.build();

		HttpComponentsClientHttpRequestFactory factory = new HttpComponentsClientHttpRequestFactory();
		factory.setHttpClient(httpClient);
		factory.setConnectTimeout(10000);
		factory.setReadTimeout(10000);
		RestTemplate template = new RestTemplate(factory);
		HttpHeaders reqHeaders = new HttpHeaders();
		reqHeaders.set("Content-Type", "application/json");
		if (headers.get("Authorization").get(0) == null)
			throw new ResponseStatusException(HttpStatus.UNAUTHORIZED);
		reqHeaders.set("Authorization", headers.get("Authorization").get(0));
		HttpEntity<String> entity = new HttpEntity<String>(body.toString(), reqHeaders);
		try {
			answer = template.postForObject(KAKAO_SERVER_EVENT, entity, String.class);
			logger.info(answer);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return ResponseEntity.status(HttpStatus.OK).body(answer);
	}

	@RequestMapping(value = "/events/token", method = RequestMethod.POST)
	public @ResponseStatus @ResponseBody HttpEntity<?> FromValleyServerToken(@RequestHeader HttpHeaders headers,
			@RequestParam(value = "refresh_token", required = true) String refresh_token,
			@RequestParam(value = "grant_type", required = true) String grant_type, HttpEntity<String> httpEntity,
			HttpServletResponse response) {
		logger.info(refresh_token);
//		String KAKAO_SERVER_EVENT = "https://kakaohome-api.i.kakao.com/events/token";
		// beta version
//		String KAKAO_SERVER_EVENT = "https://kakaohome-api.i.kakao.com/events/token";
		String answer = "";

		MultiValueMap<String, String> multiValueMap = new LinkedMultiValueMap<>();
		multiValueMap.add("refresh_token", refresh_token);
		multiValueMap.add("grant_type", grant_type);

		CloseableHttpClient httpClient = HttpClients.custom().setSSLHostnameVerifier(new NoopHostnameVerifier())
				.build();

		HttpComponentsClientHttpRequestFactory factory = new HttpComponentsClientHttpRequestFactory();
		factory.setHttpClient(httpClient);
		factory.setConnectTimeout(10000);
		factory.setReadTimeout(10000);

		RestTemplate template = new RestTemplate(factory);

		HttpHeaders header = new HttpHeaders();
		header.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
		header.add("Authorization", headers.get("Authorization").get(0));

		HttpEntity<MultiValueMap<String, String>> entity = new HttpEntity<>(multiValueMap, header);

		logger.info(entity.toString());

		answer = template.postForObject(KAKAO_SERVER_EVENT + "/token", entity, String.class);

		return ResponseEntity.status(HttpStatus.OK).body(answer);
	}

	@RequestMapping(value = "/aws/healthcheck", method = RequestMethod.GET)
	public void healthCheck(HttpEntity<String> httpEntity) {

		return;
	}

}
